;********************************************************************************
;********************************************************************************
;
; 
; Bibliothek zum Verwenden eines externen I2C-Bausteins 
; (Header Datei: i2c_inc_h.asm)
; source: i2c_inc.asm 
; Rev. 1.4 von 2010-01-04
; 
;
;********************************************************************************
;********************************************************************************
;
; Vorbereitungen in jedem Hauptprogramm, das die Bibliotheksroutinen nutzt
;********************************************************************************
;	.equ XTAL = 16000000		; Quarztaktfrequenz in Hertz
;	.equ takt = xtal			; for compatibility sake
;	.include "m8def.inc"		; Controller Definitionsfile einbinden

;.include "i2c_inc_h.asm"	; Header fr die i2c_eeprom Bibliothek
;.include "rs232inc_header.asm"	; Header fr die V24-Schnittstelle
;.include "arithmetik_header.asm"; Header fr Mathe-Modul

; Deklarationen
;	.equ mega8 = 1
;
; Interface
; ###############################################################################
; Proceduren
;	Name				Input	Output		Affected 	Bemerkung
; -------------------------------------------------------------------------------
; i2c_init				-		-			-			Setzt Presc. und Bitrate
; i2c_eeread			tw_addrl R16		R16			Eingelesenes Byte wird 
;						tw_addrh -			-			in R16 zurckgeben
; i2c_eewrite			tw_data	-			-			Datenbyte in R16 wird in 
;						tw_addrl-			-			Zelle Adresse low
;						tw_addrh-			-			Adresse high geschrieben
;						TW_HWADR-			-			Hardwareadresse
; i2c_eePwrite			tw_addrl -			-			lscht Speicher (Pages)
;						tw_addrh
;						tw_HWADR
;						tw_buffptr  					Anzahl Zeichen 0..0x1F=32	
;											tw_buffptr	0 = OK; >0 = Error		
; i2c_read				R17		-			-			Lesen eines Bytes von 
;						R16		-			-			Portexpander R16=CSW/R
; i2c_write				R17		-			-			Datenbyte auf Portexpand.
;						R16		-			-			schreiben, R16=CSW/R
; i2c_clear				tw_last R16			R16			lscht Speicher (einzeln)
;						tw_HWADR						tw_last=hchste adresse+1
; i2c_erase_all			tw_addrl -			-			lscht Speicher (Pages)
;						tw_addrh
;						tw_HWADR
;						(Ausbau fr eine Reihe von Pages ab tw_addrl:tw_addrh)
; ...............................................................................
; interne Proceduren							
; i2c_wait				r17		-			-			wartet ca r17*775 Takte
; i2c_inc_addr			-		-			-			erhht EEPROM-Zellen-Adr.
; i2c_wait_for_twint	-		-			-			wartet auf TW-INT-Bit
; i2c_Start				-		-			-			setzt START-Cond. auf Bus
; i2c_Start_Repeated	-		-			-			setzt wiederholte START-C
; i2c_Stop				-		-			-			setzt STOP-Cond. auf Bus
; i2c_send_csw			R16		-			-			sendet Chipselect+Write
; i2c_send_csr			R16		-			-			sendet Chipselect+Read
; i2c_send_byte			R16		-			-			sendet ein bel. Byte 
; i2c_receive_byte		-		R16			R16			empfngt ein Byte von i2c
; i2c_receive_continous	R16		R16			R16			empfngt Bytes mit ACK
;						R16								0 = ohne; 1 = mit ACK
; -------------------------------------------------------------------------------


; Variablen im SRAM	
;	tw_action:	.byte 1				; Merker fr IRQ-Handler
;		.equ fstart = 0				; Flag fr Start Condition transmitted
;		.equ fsla_w = 1				; Flag fr Slave Address + Writeflag (0) ges.
;		.equ fsla_r = 2				; Flag fr Slave Address + Readflag (1) ges.
;		.equ fstop  = 3				; Flag fr Stop Condition gesendet
;
;	tw_error: 	.byte 1				; TW-Fehlermeldung fr aufrufendes Programm
;  	tw_hwadr:	.byte 1				; Chipselect + HW + R/W - Adresse                     
;	tw_data:	.byte 1				; zu sendendes oder empfangenes Byte
;	tw_wait:	.byte 1				; Schleifenwert fr Wartezyklus
;	tw_last:	.byte 1				; hchste Zellenadresse des eeproms (High)
; 	tw_addr:
;	tw_addrl:	.byte 1
;	tw_addrh:	.byte 1				; EEPROM Byte-Adresse
;	tw_status: 	.byte 1				; Was luft gerade mit dem Externen EEPROM?
;		.equ fee_lesen 		= 0		; Flagbit
;		.equ fee_schreiben 	= 1		; Flagbit
;	tw_buffer:	.BYTE 256			; 256 Byte Schreib/Lese-Puffer fr EEPROM
;	tw_buffptr:	.byte 1				; Zeiger in den Schreibpuffer

; ###############################################################################
; *******************************************************************************
;
; Folgende Werte fr die Variable "debuglevel" sind in Gebrauch:
; 	 Ausgaben in binrer Form oder hexadezimal ber RS232
; 0: Release, kein Debugging
; 1: eingehender Test der EEPROM-Funktionen mit Ausgabe von TWCR und TWSR
; 2: Untersuchung der EEPROM-Adressen, die von "i2c_eeprom.asm" verwendet werden
; 3: Untersuchung der EEPROM-Adressen direkt in der Prozedur "i2c_inc_addr"
; 4: ...
;
; *******************************************************************************
;
; Registerbeschreibung
;
; *******************************************************************************
;
; 	Folgende Register sind im AVR fr TWI-Exchange vorhanden
;	TWBR:	enthlt die Bitrate (TWI-Frequenz) wie unten berechnet

;	TWCR:	Controllregister mit folgenden Flags:
;			TWINT: 	Interruptflag set by HWare 
;					(IRQ, wenn auch SREG.I = 1 und TWCR.TWIE = 1)
;					Lschen durch schreiben einer 1 in Bit TWINT by software
;					alle Operationen mssen vorher erledigt sein, weil durch
;					Rcksetzen das TWI aktiv wird
;			TWEA:	cknowledge Enable after receiving a Byte
;			TWSTA:	Sende START-Condition falls Bus frei; 
;					lschen durch SW nachdem START gesendet ist
;			TWSTO:	Sende STOP-Condition; wird automatisch gelscht
;			TWWC:	Write-Collision geht auf 1, wenn TWDR geschrieben wird
;					whrend TWINT = 0, cleared by writing TWDR while TWINT = 1
;			TWEN:	Enable TWI and activate TWI-Interface;TWI bernimmt Kontrolle
;					ber die Pins SCL und SDA am Controller
;			TWIE:	IRQ-Enable; (siehe TWINT)

;	TWSR:	Status-Register: Bits7:3
;			Prescaler: Bits 1:0		0b00 = 1; 0b01 = 4; 0b10 = 16; 0b11 = 64
;
;	TWDR:	Datenregister Schreiben/Lesen
;	TWAR:	Slave-Adresse Bits 7:1
;			TWGCE Bit 0: General call recognition enable
;
; *******************************************************************************
i2c_init:
	push r16
	in r16, sreg
	push r16
	in r16, TWSR					; TWSR laden und TWPS1:0 ausmaskieren
	andi r16, 0b11111100			;
	ori r16, (tw_prescaler & 0b00000011); Prescalerbits setzen
	out twsr, r16					; nach TWSR zurckschreiben
	ldi r16, tw_bitrate				; Bitrate-Teiler 
	out twbr, r16					; nach twbr schreiben
	pop r16
	out sreg, r16
	pop r16
	ret


i2c_eeread:							; (tw_data=data; tw_addrl/h=ee-Zelle; 
	push r16						;  HWAdr in TW_hwadr)
	in r16, sreg
	push r16						; 
	rcall i2c_start					; start-condition senden
;	ldi r16,'A'
;	rcall serout
	lds r16, tw_hwadr				; Chipselect (Slaveadresse) + Write
	rcall i2c_send_csw				; senden
;	ldi r16,'B'
;	rcall serout
	lds r16, tw_addrh				; EEPROM-Adresse (high)
	rcall i2c_send_byte				; senden
;	ldi r16,'C'
;	rcall serout
	lds r16, tw_addrl				; EEPROM-Adresse (low)
	rcall i2c_send_byte				; senden
;	ldi r16,'D'
;	rcall serout
	rcall i2c_start_repeated		; repeated start-condition senden
;	ldi r16,'E'
;	rcall serout
	lds r16, tw_hwadr 				; Chipselect (Slaveadresse) + Read
	ori r16, 0x01 
	rcall i2c_send_csr				; senden
;	ldi r16,'F'
;	rcall serout
	rcall i2c_receive_byte			; Warten auf Byte
	sts tw_data, r16
;	ldi r16,'G'
;	rcall serout
	rcall i2c_stop					; stop-condition senden
;	ldi r16,'H'
;	rcall serout
	pop r16
	out sreg, r16
	pop r16
	ret

i2c_eewrite:					; (tw_data=data; tw_addrl/h=ee-Zelle; 
	push r16					;  HWAdr in TW_hwadr)
	in r16, sreg
	push r16
	push r17
	rcall i2c_start				; start-condition senden
	.if debuglevel==1
		ldi r16,'-'
		rcall serout
		in r16, twcr
		rcall v24_number_bin
		rcall v24_blank
		in r16, twsr
		rcall v24_number_bin
		rcall v24_blank
	.endif
	lds r16, tw_hwadr			; Chip und Hardwareadresse holen
	rcall i2c_send_csw			; CSW ist vor Aufruf der Procedur i2c_eewrite
								; in TW_hwadr abzulegen
	.if debuglevel==1
		ldi r16,'a'
		rcall serout
	.endif
	lds r16, tw_addrh			; EEPROM-Adresse (high)
	rcall i2c_send_byte
	.if debuglevel==1
		ldi r16,'b'
		rcall serout
	.endif
	lds r16, tw_addrl			; EEPROM-Adresse (low)
	rcall i2c_send_byte
	.if debuglevel==1
		ldi r16,'c'
		rcall serout
	.endif
	lds r16, tw_data			; Datenbyte senden
	rcall i2c_send_byte
	.if debuglevel==1
		ldi r16,'d'
		rcall serout
	.endif
	rcall i2c_stop				; stop-condition senden
	.if debuglevel==1
		ldi r16,'e'
		rcall serout
		rcall v24_lfcr
	.endif
	lds R17, tw_wait
	rcall i2c_warte
	pop r17
	pop r16
	out sreg, r16
	pop r16
	ret
;
;
; ------------------------------------------------------------------------------------
;
i2c_eePwrite:					; (Daten in tw_buffer; tw_addrl/h=ee-Zelle; 
	push r16					;  HWAdr in TW_hwadr; anzahl Bytes in tw_buffptr)
	in r16, sreg
	push r16
	push r17
	push xl
	push xh
	lds r17, tw_buffptr
	andi r17, 0b00011111		; Anzahl muss < 32 sein
	sts tw_buffptr, r17			; Fehler, wenn Rckgabewert in tw_buffptr = 0!!
	tst r17 
	breq i2c_eePwrite_done
	inc r17
	ldi xl,  low(tw_buffer)		; Pufferadresse low
	ldi xh, high(tw_buffer)		; und High ins Indexregisterpaar xl:xh
	rcall i2c_start				; start-condition senden
	.if debuglevel==5
		ldi r16,'-'
		rcall serout
		in r16, twcr
		rcall v24_number_bin
		rcall v24_blank
		in r16, twsr
		rcall v24_number_bin
		rcall v24_blank
	.endif
	lds r16, tw_hwadr			; Chip und Hardwareadresse holen
	rcall i2c_send_csw			; CSW ist vor Aufruf der Procedur i2c_eewrite
								; in TW_hwadr abzulegen
	.if debuglevel==5
		ldi r16,'a'
		rcall serout
	.endif
	lds r16, tw_addrh			; EEPROM-Adresse (high)
	rcall i2c_send_byte
	.if debuglevel==5
		ldi r16,'b'
		rcall serout
	.endif
	lds r16, tw_addrl			; EEPROM-Adresse (low)
	rcall i2c_send_byte
	.if debuglevel==5
		ldi r16,'c'
		rcall serout
	.endif
i2c_eePwrite_01:
	ld r16, x+					; Datenbyte aus Puffer laden, dann index erhhen
	rcall i2c_send_byte			; senden
	.if debuglevel==5
		ldi r16,'d'
		rcall serout
	.endif
	dec r17						; Bytezhler herunterzhlen
	brne i2c_eePwrite_01		; 
	rcall i2c_stop				; stop-condition senden
	.if debuglevel==5
		ldi r16,'e'
		rcall serout
		rcall v24_lfcr
	.endif
	lds R17, tw_wait
	rcall i2c_warte

i2c_eePwrite_done:
	pop xh
	pop xl
	pop r17
	pop r16
	out sreg, r16
	pop r16
	ret
;
;
; ------------------------------------------------------------------------------------
;
i2c_read:						; HW_adr => R16; R17 <= Data; 
	push r18					;  
	in r18, sreg
	push r18
	push r16

	rcall i2c_start					; start-condition senden
	.if debuglevel == 6
		mov r18, r16
		ldi r16,'A'
		rcall serout
		mov r16, r18
	.endif
	.if debuglevel == 6				; Das doppelte Einfgen dieser Testsequenz hat 
		mov r18, r16				; gezeigt, dass der Hnger an dieser Stelle ein
		in r16, twcr				; zeitliches Problem war. Das noch nicht gesetzte
		rcall v24_number_bin		; TWINT-Bit in TWCR hat bei der folgenden Ausgabe
		rcall v24_blank				; der CSR zu einer Writecollision gefhrt (TWWC=1)
		in r16, twsr				; 
		rcall v24_number_bin
		rcall v24_lfcr
		ldi r16,'B'
		rcall serout
		mov r16, r18
	.endif
	rcall i2c_wait_for_twint		; das Einfgen dieser Routine hat das Problem 
	ori r16, 0x01 					; behoben
	.if debuglevel == 6
		mov r18, r16
		rcall v24_blank
		in r16, twcr
		rcall v24_number_bin
		rcall v24_blank
		in r16, twsr
		rcall v24_number_bin
		rcall v24_lfcr
		ldi r16,'B'
		rcall serout
		mov r16, r18
	.endif
	rcall i2c_send_csr				; senden
	rcall i2c_receive_byte			; Warten auf Eingabe-Byte
	mov r17, r16					; in richtige Register schieben
	.if debuglevel == 6
		ldi r16,'C'
		rcall serout
	.endif
	rcall i2c_stop					; stop-condition senden
	.if debuglevel == 6
		ldi r16,'D'
		rcall serout
	.endif

	pop r16
	pop r18
	out sreg, r18
	pop r18

	ret

;
; ------------------------------------------------------------------------------------
;
i2c_write:						; (r16 = HW_Adresse; r17=data (wird nicht verndert))
	push r16
	push r18					; 
	in r18, sreg
	push r18
	rcall i2c_start				; start-condition senden
	.if debuglevel==5
		mov r18, r16				; r16 gegen Venderungen sichern
		ldi r16,'-'
		rcall serout
		in r16, twcr
		rcall v24_number_bin
		rcall v24_blank
		in r16, twsr
		rcall v24_number_bin
		rcall v24_blank
		mov r16, r18			; r16 fr Datenbergabe restaurieren
	.endif
	rcall i2c_send_csw			; CSW ist vor Aufruf der Procedur i2c_eewrite
								; in R16 abzulegen
	.if debuglevel==5
		ldi r16,'a'
		rcall serout
	.endif
	mov r16, r17				; Datenbyte senden
	rcall i2c_send_byte
	.if debuglevel==5
		ldi r16,'b'
		rcall serout
	.endif
	rcall i2c_stop				; stop-condition senden
	.if debuglevel==5
		ldi r16,'c'
		rcall serout
		rcall v24_lfcr
	.endif
;	lds R17, tw_wait
;	rcall i2c_warte
	pop r18
	out sreg, r18
	pop r18
	pop R16
	ret

;
; ------------------------------------------------------------------------------------
;
i2c_clear:						; CSW incl Chipadr. und W-Bit in tw_hwadr
	push r16					; Zellenzeiger in tw_addrl:tw_addrh 
	push r17
	in r17, sreg
	push r17
	push r18
	lds r18, tw_last
	ldi r16, 0xff
	sts tw_data, r16			; Zelle ist gelscht, wenn alle Bits auf 1 sind
	ldi r17, 0					; 0x0000 als startadresse
	sts tw_addrl, r17
	sts tw_addrh, r17
i2c_clear01:
	.if debuglevel == 4
		lds r16, tw_addrh
		rcall v24_number_hex
		rcall v24_lfcr
		ldi r16, 0
	.endif
i2c_clear02:
	sts tw_addrl, r16
	rcall i2c_eewrite
	inc r16
	.if debuglevel == 4
		rcall v24_number_hex
		rcall v24_blank
	.endif
	brne i2c_clear02
	inc r17
	sts tw_addrh, r17
	cp  r18, r17
	brne i2c_clear01
	pop r18
	pop r17
	out sreg, r17
	pop r17
	pop r16
	ret

;
; ------------------------------------------------------------------------------------
;
i2c_erase_all:					; CSW incl Chipadr. und W-Bit in tw_hwadr
	push xl						; Zellenzeiger in tw_addrl:tw_addrh 
	push xh					
	push r14
	push r15
	push r16
	push r17
	in r17, sreg
	push r17
	ldi xl,  low(tw_buffer)		; Buffer im SRAM fllen
	ldi xh, high(tw_buffer)		; Adresse von tw_buffer als Index
	ldi r16, 0					; 0 als Durchlaufzhler und speichern in
	mov r15, r16				; Hilfsregister fr Addition
	mov r14, r16				; und als Zhler fr Pagewrite
	ldi r17, 0xFF				; Lschen = alle Bits auf 1
i2c_erase_all_b:
	st x+, r17					; in tw_buffer ablegen und incrementieren
	inc r16						; Buffer-Zeiger erhhen
	cpi r16, 0x20				; schon 32 erreicht?
	brne i2c_erase_all_b		; nein, dann noch einmal
	dec r16						; Positionen von 0x00 bis 0x1F besetzt
	sts tw_buffptr, r16			; 32 Byte im Puffer
i2c_erase_all_c:
	rcall i2c_eePwrite			; Inhalt von SRAM nach EEPROM-Puffer schreiben
	lds r16, tw_addrl			; alten Speicherzeiger LOW laden
	subi r16, -32				; addiere 32
	sts tw_addrl, r16			; zurckschreiben
	lds r16, tw_addrh			; dto HIGH
	adc r16, r15				; 0 mit Carry addieren
	sts tw_addrh, r16			; zurckschreiben
	ldi r16, 0x1f				; Buffer Pointer als Zhler neu setzen
	sts tw_buffptr, r16			; 
	dec r14						; Durchgangszhler (256 mal 32 = 8192 = 2KB)
	brne i2c_erase_all_c		; bis 0 herunterzhlen (=256 Speicherseiten)
	pop r17						; fertig
	out sreg, r17
	pop r17
	pop r16
	pop r15
	pop r14
	pop xh
	pop xl
	ret
;
; ------------------------------------------------------------------------------------
;
i2c_warte:						; i2c_outerloop-Wert wird in R17 bergeben
	push r16
	push r18
	in r18, sreg
i2c_warte_outer:
	ldi r16, i2c_innerloop			; 1 T
i2c_warte_inner:
	dec r16						; 1 T
	nop							; 1 T
	brne i2c_warte_inner		; 2/1 T -> i2c_iduration = 4 * i2c_innerloop - 1
	dec r17						; 1 T
	brne i2c_warte_outer		
	; 2/1 T -> i2c_oduration = i2c_outerloop * (4+i2c_iduration)-1
	; i2c_oduration = 8 ms = 80000 * xtal / 1000000 T (-12* T)
	; i2c_innerloop = 196 (aus Faktorzerlegung)
	; i2c_outerloop = i2c_oduration / (4*i2c_innerloop - 1) 
	out sreg, r18
	pop r18
	pop r16
	ret
;
; ------------------------------------------------------------------------------------
;
i2c_inc_addr:
	push r16
	in r16, sreg
	push r16
	push XL
	push XH
	.if debuglevel == 3
		lds r16, tw_addrh
		rcall v24_number_hex
		lds r16, tw_addrl
		rcall v24_number_hex
		ldi r16, ':'
		rcall serout
	.endif
	lds xl, tw_addrl
	lds xh, tw_addrh
	adiw xl, 0x01
	sts tw_addrl, xl
	sts tw_addrh, xh
	.if debuglevel == 3
		lds r16, tw_addrh
		rcall v24_number_hex
		lds r16, tw_addrl
		rcall v24_number_hex
		rcall v24_blank
	.endif
	pop XH
	pop XL
	pop r16
	out sreg, r16
	pop r16
	ret

;
; ------------------------------------------------------------------------------------
;
i2c_wait_for_twint:					; auf das TWINT-Flag warten
	push r17
	in r17,sreg
	push r17
wftwint:
	in r17, twcr					; Control-Register lesen
	sbrs r17, twint					; RJMP berspringen, wenn twint = 1
	rjmp wftwint					; warten bis abgeschlossen (twint war 0)
	pop r17
	out sreg,r17
	pop r17
	ret

;
; ------------------------------------------------------------------------------------
;
i2c_start:
	push r16						; r16 sichern
	in r16, sreg
	push r16
	ldi r16, (1<<TWint) | (1<<TWsta) | (1<<TWen); Bits fr Startcondition, TW enabel 
	out TWcr, r16					; und action -> START-condition senden
i2c_start_wait_ack:	
	in r16, TWsr					; tw-Status lesen
	andi r16, tw_stmask				; Prescalerbits ausmaskieren
	cpi r16, tw_start				; test auf "start gesendet"
	brne i2c_start_wait_ack			; noch nicht, dann warten
	pop r16
	out sreg, r16
	pop r16							; r16 restaurieren
	ret
;
i2c_start_repeated:
	push r16						; r16 sichern
	in r16, sreg
	push r16
	ldi r16, (1<<TWint) | (1<<TWsta) | (1<<TWen); Bits fr Startcondition, TW enabel 
	out TWcr, r16					; und action -> START-condition senden
i2c_start_rep_wait_ack:	
	in r16, TWsr					; tw-Status lesen
	andi r16, tw_stmask				; Prescalerbits ausmaskieren
	cpi r16, tw_rep_start			; test auf "repeated start gesendet"
	brne i2c_start_rep_wait_ack		; noch nicht, dann warten
	pop r16
	out sreg, r16
	pop r16							; r16 restaurieren
	ret
;
i2c_stop:
	push r16						; r16 sichern
	in r16, sreg
	push r16
	ldi r16, (1<<TWint) | (1<<TWsto) | (1<<TWen); Bits fr Stopcondition, TW enabel 
	out TWcr, r16					; und action -> STOP*-condition senden
i2c_stop_wait:	
	in r16, TWcr					; tw-Status lesen
	andi r16, (1<<twsto)			; Prescalerbits ausmaskieren
	brne i2c_stop_wait				; noch nicht, dann warten
	pop r16
	out sreg, r16
	pop r16							; r16 restaurieren
	ret

;
; -------------------------------------------------------------------------------	
;
i2c_send_csw:
;	Chipkennung/HW-Adr+W wird in R16 bergeben
	
	rcall i2c_wait_for_twint
	.if debuglevel == 1
		push r16
		ldi r16,':'
		rcall serout
		in r16, twcr
		rcall v24_number_bin
		pop r16
	.endif
	out twdr, r16					; setzen des CSW-Bytes mit der CHip+HW-Adresse+R/~W
	.if debuglevel == 1
		push r16
		rcall v24_blank
		in r16, twsr
		rcall v24_number_bin
		pop r16
	.endif
	push r16						; Chipselect und HW-Adr. in R16
	in r16, sreg
	push r16
	ldi r16, (1<<TWint) | (1<<TWen)	; Bits fr, TW enabel und TW ein
	out TWcr, r16					; Ausgabe und los
	.if debuglevel == 1
		ldi r16,'_'
		rcall serout
	.endif
	rcall i2c_wait_for_twint		; Warten auf TWint=1
	.if debuglevel == 1
		in r16, twcr
		rcall v24_number_bin
		ldi r16,'.'
		rcall serout
		rcall v24_blank
	.endif
i2c_send_csw_wait_ack:
	in r16, TWsr					; tw-Status lesen
	andi r16, tw_stmask				; Prescalerbits ausmaskieren
	.if debuglevel == 1
		rcall v24_number_bin
		rcall v24_lfcr
	.endif
	cpi r16, mt_sla_ack				; test auf "SLA+W"
	breq i2c_send_csw_wait_go_on	; noch nicht, dann warten
	cpi r16, mt_sla_nack
	brne i2c_send_csw_wait_ack
i2c_send_csw_wait_go_on:
	pop r16
	out sreg, r16
	pop r16
	ret
;
; -------------------------------------------------------------------------------	
;
i2c_send_csr:
;	Chipkennung/HW-Adr+R wird in R16 bergeben
;	rcall i2c_wait_for_twint
;	ldi r16, '>'
;	rcall serout
;	in r16, twcr
;	rcall v24_number_bin
;	ldi r16, csr
	out twdr, r16					; setzen des CSW-Bytes mit der CHip+HW-Adresse+R/~W
	push r16						; Chipselect und HW-Adr. in R16
	in r16, sreg
	push r16
	ldi r16, (1<<TWint) | (1<<TWen)	; Bits fr, TW enabel und TW ein
	out TWcr, r16					; Ausgabe und los
;	ldi r16, '_'
;	rcall serout
;	in r16, twcr
;	rcall v24_number_bin
	rcall i2c_wait_for_twint		; Warten auf TWint=1
;	ldi r16, '-'
;	rcall serout
;	in r16, twsr
;	rcall v24_number_bin
;	ldi r16, ':'
;	rcall serout
i2c_send_csr_wait_ack:
	in r16, TWsr					; tw-Status lesen
	andi r16, tw_stmask				; Prescalerbits ausmaskieren
;	rcall v24_number_bin
;	rcall v24_lfcr
	cpi r16, mr_sla_ack				; test auf "SLAdr+R geschrieben, ACK empf."
	brne i2c_send_csr_wait_ack		; noch nicht, dann warten
;	in r16, twsr
;	rcall v24_number_bin
	pop r16
	out sreg, r16
	pop r16
	ret
;
; -------------------------------------------------------------------------------	
;
i2c_send_byte:
;	zu sendendes Byte ist in R16
	out twdr, r16					; Byte ins Senderegister schreiben
	.if debuglevel==1
		rcall v24_lfcr
		rcall v24_number_hex
		rcall v24_blank
	.endif
	in r16, sreg
	push r16
	ldi r16, (1<<TWint) | (1<<TWen)	; Bits fr, TW enabel und TW ein
	out TWcr, r16					; Ausgabe ins Controlregister und los
	.if debuglevel==1
		rcall v24_blank
		ldi r16,'A'
		rcall serout
		rcall v24_number_bin
		rcall v24_blank
	.endif
i2c_send_byte_wait_twint:			; auf das TWINT-Flag warten
	in r16, twcr					; Control-Register lesen
	.if debuglevel==1
		rcall v24_number_bin
		rcall v24_blank
	.endif
	sbrs r16, twint					; RJMP berspringen, wenn twint = 1
	rjmp i2c_send_byte_wait_twint	; warten bis abgeschlossen (twint war 0)
	.if debuglevel==1
		ldi r16,'B'
		rcall serout
		rcall v24_blank
	.endif
i2c_send_byte_wait_ack:
	in r16, TWsr					; tw-Status lesen
	andi r16, tw_stmask				; Prescalerbits ausmaskieren
	.if debuglevel==1
		rcall v24_number_bin
		rcall v24_blank
	.endif
	cpi r16, mt_data_ack			; test auf "byte gesendet ACK empfangen"
	breq i2c_send_byte_wait_ack_go_on; noch nicht, dann warten
	cpi r16, mt_data_nack
	brne i2c_send_byte_wait_ack
i2c_send_byte_wait_ack_go_on:
	.if debuglevel==1
		rcall v24_number_bin		; TWSR kontrollieren
		rcall v24_lfcr
	.endif
	pop r16
	out sreg, r16
	ret
;
; -------------------------------------------------------------------------------	

i2c_receive_byte:
	in r16, sreg
	push r16
	ldi r16, (1<<TWint) | (1<<TWen)	; TW enabel und TW ein um Byte zu empfangen
	out twcr, r16					; Empfang anstoen
i2c_receive_byte_wait_twint:
	in r16, twcr					; Control-Register lesen
	sbrs r16, twint					; RJMP berspringen, wenn twint = 1
	rjmp i2c_receive_byte_wait_twint; warten bis abgeschlossen (twint war 0)
i2c_receive_byte_ready:
	in r16, TWsr					; tw-Status lesen
	andi r16, tw_stmask				; Prescalerbits ausmaskieren
	cpi r16, mr_data_nack			; test auf "BYTE EMPFANGEN NACK"
	brne i2c_receive_byte_ready		; noch nicht, dann warten
	pop r16
	out sreg, r16
	in r16, twdr					; Datenbyte laden
	; Rckgabe des empfangenen Bytes in R16
	ret
	

