;********************************************************************************
;********************************************************************************
; Headerdatei zur
; Bibliothek zum Verwenden externer I2C-Bausteine
; Header Datei: i2c_inc_h.asm
; (source: i2c_inc.asm )
; Rev. 1.4 von 2010-01-04
; 
; Interface
; ###############################################################################
; Proceduren
;	Name				Input	Output		Affected 	Bemerkung
; -------------------------------------------------------------------------------
; i2c_init				-		-			-			Setzt Presc. und Bitrate
; i2c_eeread			tw_addrl R16		R16			Eingelesenes Byte wird 
;						tw_addrh -			-			in R16 zurckgeben
; i2c_eewrite			tw_data	-			-			Datenbyte in R16 wird in 
;						tw_addrl-			-			Zelle Adresse low
;						tw_addrh-			-			Adresse high geschrieben
;						TW_HWADR-			-			Hardwareadresse
; i2c_eePwrite			tw_addrl -			-			lscht Speicher (Pages)
;						tw_addrh
;						tw_HWADR
;						tw_buffptr  					Anzahl Zeichen 0..0x1F=32	
;											tw_buffptr	0 = OK; >0 = Error		
; i2c_read				R17		-			-			Lesen eines Bytes von 
;						R16		-			-			Portexpander R16=CSW/R
; i2c_write				R17		-			-			Datenbyte auf Portexpand.
;						R16		-			-			schreiben, R16=CSW/R
; i2c_clear				tw_last R16			R16			lscht Speicher (einzeln)
;						tw_HWADR						tw_last=hchste adresse+1
; i2c_erase_all			tw_addrl -			-			lscht Speicher (Pages)
;						tw_addrh
;						tw_HWADR
;						(Ausbau fr eine Reihe von Pages ab tw_addrl:tw_addrh)
; ...............................................................................
; interne Proceduren							
; i2c_wait				r17		-			-			wartet ca r17*775 Takte
; i2c_inc_addr			-		-			-			erhht EEPROM-Zellen-Adr.
; i2c_wait_for_twint	-		-			-			wartet auf TW-INT-Bit
; i2c_Start				-		-			-			setzt START-Cond. auf Bus
; i2c_Start_Repeated	-		-			-			setzt wiederholte START-C
; i2c_Stop				-		-			-			setzt STOP-Cond. auf Bus
; i2c_send_csw			R16		-			-			sendet Chipselect+Write
; i2c_send_csr			R16		-			-			sendet Chipselect+Read
; i2c_send_byte			R16		-			-			sendet ein bel. Byte 
; i2c_receive_byte		-		R16			R16			empfngt ein Byte von i2c
; i2c_receive_continous	R16		R16			R16			empfngt Bytes mit ACK
;						R16								0 = ohne; 1 = mit ACK
; -------------------------------------------------------------------------------

; Variablen im SRAM	
;	tw_action:	.byte 1				; Merker fr IRQ-Handler
;		.equ fstart = 0				; Flag fr Start Condition transmitted
;		.equ fsla_w = 1				; Flag fr Slave Address + Writeflag (0) ges.
;		.equ fsla_r = 2				; Flag fr Slave Address + Readflag (1) ges.
;		.equ fstop  = 3				; Flag fr Stop Condition gesendet
;
;	tw_error: 	.byte 1				; TW-Fehlermeldung fr aufrufendes Programm
;  	tw_hwadr:	.byte 1				; Chipselect + HW + R/W - Adresse                     
;	tw_data:	.byte 1				; zu sendendes oder empfangenes Byte
;	tw_wait:	.byte 1				; Schleifenwert fr Wartezyklus
;	tw_last:	.byte 1				; hchste Zellenadresse des eeproms (High)
; 	tw_addr:
;	tw_addrl:	.byte 1
;	tw_addrh:	.byte 1				; EEPROM Byte-Adresse
;	tw_status: 	.byte 1				; Was luft gerade mit dem Externen EEPROM?
;		.equ fee_lesen 		= 0		; Flagbit
;		.equ fee_schreiben 	= 1		; Flagbit
;	tw_buffer:	.BYTE 256			; 256 Byte Schreibpuffer fr EEPROM
;	tw_buffptr:	.byte 1				; Zeiger in den Schreibpuffer

; ###############################################################################
; *******************************************************************************
;
; Folgende Werte fr die Variable "debuglevel" sind in Gebrauch:
; 	 Ausgaben in binrer Form oder hexadezimal ber RS232
; 0: Release, kein Debugging
; 1: eingehender Test der EEPROM-Funktionen mit Ausgabe von TWCR und TWSR
; 2: Untersuchung der EEPROM-Adressen, die von "i2c_eeprom.asm" verwendet werden
; 3: Untersuchung der EEPROM-Adressen direkt in der Prozedur "i2c_inc_addr"
; 4: ...
;
; *******************************************************************************
;
; Registerbeschreibung
;
; *******************************************************************************
;
; 	Folgende Register sind im AVR fr TWI-Exchange vorhanden
;	TWBR:	enthlt die Bitrate (TWI-Frequenz) wie unten berechnet

;	TWCR:	Controllregister mit folgenden Flags:
;			TWINT: 	Interruptflag set by HWare 
;					(IRQ, wenn auch SREG.I = 1 und TWCR.TWIE = 1)
;					Lschen durch schreiben einer 1 in Bit TWINT by software
;					alle Operationen mssen vorher erledigt sein, weil durch
;					Rcksetzen das TWI aktiv wird
;			TWEA:	cknowledge Enable after receiving a Byte
;			TWSTA:	Sende START-Condition falls Bus frei; 
;					lschen durch SW nachdem START gesendet ist
;			TWSTO:	Sende STOP-Condition; wird automatisch gelscht
;			TWWC:	Write-Collision geht auf 1, wenn TWDR geschrieben wird
;					whrend TWINT = 0, cleared by writing TWDR while TWINT = 1
;			TWEN:	Enable TWI and activate TWI-Interface;TWI bernimmt Kontrolle
;					ber die Pins SCL und SDA am Controller
;			TWIE:	IRQ-Enable; (siehe TWINT)

;	TWSR:	Status-Register: Bits7:3
;			Prescaler: Bits 1:0		0b00 = 1; 0b01 = 4; 0b10 = 16; 0b11 = 64
;
;	TWDR:	Datenregister Schreiben/Lesen
;	TWAR:	Slave-Adresse Bits 7:1
;			TWGCE Bit 0: General call recognition enable
;
; *******************************************************************************
;	Definition der Statusmeldungen
; *******************************************************************************
;
;	 AVR-specific Stuff
	.equ TW_StMask	  	= 0xF8	; TWI-Statusmaske (Bits 7:3)
; 	 Statusmeldungen in TWSR.7:3	
	.equ TW_START		= 0x08	; Start Condition wurde gesendet
	.equ TW_REP_START	= 0x10	; Repeated Start Condition wurde gesendet
	.equ MT_SLA_ACK 	= 0x18	; SLA+W gesendet, Acknowledge empfangen
	.equ MT_SLA_NACK	= 0x20	; SLA+W gesendet, NOTAcknowledge empfangen
	.equ MT_DATA_ACK 	= 0x28	; Data Byte gesendet, Acknowledge empfangen
	.equ MT_DATA_NACK	= 0x30	; Data Byte gesendet, NOTAcknowledge empfangen
	.equ MT_ARB_LOST	= 0x38	; Arbitration lost in SLA+W oder Databytes
	.equ MR_ARB_LOST	= 0x38	; Arbitration lost in SLA+R oder Databytes
	.equ MR_SLA_ACK		= 0x40	; SLA+R gesendet, Acknowledge empfangen
	.equ MR_SLA_NACK	= 0x48	; SLA+R gesendet, NOTAcknowledge empfangen
	.equ MR_DATA_NACK	= 0x58	; Data Byte empfangen, NOTAcknowledge gesendet
	.equ MR_DATA_ACK	= 0x50	; Data Byte empfangen, Acknowledge gesendet
; 	 weitere Werte f. Slave-Operationen:C:\WinAVR-20081205\avr\include\util\twi.h

; *******************************************************************************
; Bausteinadressierung
; *******************************************************************************
;
; Die folgenden Konstanten werden in der Bibliothek lediglich definiert, im 
; Programmteil der Bibliothek aber nicht verwendet. Die Wertzuweisung  auf die 
; Konstanten CSR und CSW erfolgen incl. des Hardwareadressteils im jeweiligen 
; Hauptprogramm vor den Aufrufen i2c_send_CSW und i2c_send_CSR oder direkt an R16
; Beispiel: 
;	.equ CSW 	= SLA_W_24C64	; CHIP-Select fr EEPROM 24C64 auf HW-Adresse 000
;	.equ CSR 	= CSW | 0x01	; CSR = CSW mit gesetztem Bit 0
;	ldi r16, csw
;	rcall i2c_send_csw
; bei mehreren I2C-Bausteinen empfiehlt sich die Definition weiterer Konstanten
; *******************************************************************************
; Device Identifier: fr 24C64: 0xA0 = 0b1010 0000 (D_Ident.7:4)
; Device Adresse: fr 24C64 Adressbits A0, A1 und A2 (Pin 1 - 3) (D_Ident.3:1)
; W/R-Bit: Write = 0; Read = 1 (D_Ident.0)
	.equ D_Ident24C64 = 0xA0			;=0b1010 0000| 0b0000 000 0 | 0b0000000 0
	.equ SLA_R_24C64 = D_Ident24C64 + 1
	.equ SLA_W_24C64 = d_Ident24C64
	.equ last_24C64 = 0x2000		; hchste Zellenadresse des 24C64 +1

; Device Identifier: fr PCF8574: 0x40 = 0b0100 0000 (D_Ident.7:4)
; Device Adresse: fr PCF8574 Adressbits A0, A1 und A2 (Pin 1 - 3) (D_Ident.3:1)
; W/R-Bit: Write = 0; Read = 1 (D_Ident.0)
	.equ D_IdentPCF8574 = 0x40			;=0b0100 0000| 0b0000 000 0 | 0b0000000 0
	.equ SLA_R_PCF8574 = D_IdentPCF8574 + 1
	.equ SLA_W_PCF8574 = d_IdentPCF8574

; Device Identifier: fr PCF8574A: 0x70 = 0b0111 0000 (D_Ident.7:4)
; Device Adresse: fr PCF8574A Adressbits A0, A1 und A2 (Pin 1 - 3) (D_Ident.3:1)
; W/R-Bit: Write = 0; Read = 1 (D_Ident.0)
	.equ D_IdentPCF8574A = 0x70			;=0b0111 0000| 0b0000 000 0 | 0b0000000 0
	.equ SLA_R_PCF8574A = D_IdentPCF8574A + 1
	.equ SLA_W_PCF8574A = d_IdentPCF8574A

; Device Identifier: fr PCF8591: 0x90 = 0b1001 0000 (D_Ident.7:4)
; Device Adresse: fr PCF8574A Adressbits A0, A1 und A2 (Pin 1 - 3) (D_Ident.3:1)
; W/R-Bit: Write = 0; Read = 1 (D_Ident.0)
	.equ D_IdentPCF8591 = 0x90			;=0b1001 0000| 0b0000 000 0 | 0b0000000 0
	.equ SLA_R_PCF8591 = D_IdentPCF8591 + 1
	.equ SLA_W_PCF8591 = d_IdentPCF8591

; *******************************************************************************


; *******************************************************************************
; Berechnung der Taktrate
; *******************************************************************************

.ifndef XTAL					; Im HP wird der Prozessortakt meist  mit "takt" 
	.equ xtal = takt			; angegeben
.endif
	.equ TW_prescaler 	= 0b00	; TWPS = 0b00 => Prescaler = 1
	.equ TW_Clock 	= 100000	; TW-Taktfrequenz in Hertz
	.equ TW_BitRate = (Xtal/TW_Clock - 16)/(2*(EXP2(2*TW_Prescaler))) ;
								; (Dblatt p170) TW_BitRate gehrt in das TWBR

;	.equ i2c_outerloop = 164	; Berechnung unten
	.equ i2c_24C64 = 82			; empirisch ermittelt fr 24C64 -> 4 ms warten
	.equ i2c_innerloop = 194	; i2c_iduration=775Takte=48,4s bei xtal=16MHz
								; fest vorgegeben

.dseg 
tw_action:	.byte 1				; Merker fr IRQ-Handler
	.equ fstart = 0				; Flag fr Start Condition transmitted
	.equ fsla_w = 1				; Flag fr Slave Address + Writeflag (0) gesendet
	.equ fsla_r = 2				; Flag fr Slave Address + Readflag (1) gesendet
	.equ fstop  = 3				; Flag fr Stop Condition gesendet

tw_error: 	.byte 1				; TW-Fehlermeldung fr aufrufendes Programm
TW_hwadr:	.byte 1				; Chip+HW+R/W Adresse
tw_data:	.byte 1				; zu sendendes oder empfangenes Byte
tw_wait:	.byte 1				; Schleifenwert fr Wartezyklus
tw_last:	.byte 1				; hchste EEPROM-Zellenadresse + 1
tw_addr:
tw_addrl:	.byte 1
tw_addrh:	.byte 1				; EEPROM Byte-Adresse
tw_status: 	.byte 1				; Was luft gerade mit dem Externen EEPROM?
	.equ fee_lesen 		= 0		;
	.equ fee_schreiben 	= 1		;
tw_buffer:	.BYTE 256			; 256 Byte Schreibpuffer fr EEPROM
tw_buffptr:	.byte 1				; Zeiger in den Schreibpuffer


.cseg



